'--------------------------------------------------------------------------
'File Name	:	RSD_Hotfix_Installer.vbs
'Author      	:	Mathew Oommen, Balu
'   			CM MI HES
'Purpose	:	To use common MS Hotfixes install file, for RSD Install
'Date		: 	07/22/2008
'--------------------------------------------------------------------------
Option Explicit

Dim i : i = 0 
Dim j 
Dim x,y,z
Dim arrFileLines()
Dim objFSO
Dim objFile
Dim hotfixes, names, hotfixname
Dim oShell
Dim LogFilePath 
Dim LogFile
Dim filesys
Dim objRegExpr
Dim matches
Dim fileName
Dim cmd
Dim CmdArgs
Dim return_code
Dim Temp

Set objFSO = CreateObject("Scripting.FileSystemObject") 
Set objFile = objFSO.OpenTextFile("HotfixList.dat", 1)
Set oShell = CreateObject("WScript.Shell")



LogFilePath="c:\temp\MI_MI406-08-R.log"				'Change this to the pre-defined log file location

' Check if the Log file exists or not. If not create a new file
If objFSO.FileExists(LogFilePath) Then
	Set LogFile = objFSO.OpenTextFile(LogFilePath,8)
	
	'This is the default value written to the file
	'Change it accroding 
	WriteLog("[" &Date() & " - " & Time() & "]   Installing new MS Hotfixes...") 
	
else
	Set LogFile = objFSO.CreateTextFile(LogFilePath, True)
	WriteLog("[" &Date() & " - " & Time() & "]   Installing new MS Hotfixes...")
end if

'Read the text file
Do Until objFile.AtEndOfStream 
	Redim Preserve arrFileLines(i) 
	
	arrFileLines(i) = objFile.ReadLine 
	
	if Len(arrFileLines(i)) > 0 then 
	' Get all the hotfixes from the provided text file 
	GetAllHotFixes(arrFileLines(i)) 
	i = i + 1 
	end if 
Loop 

WriteLog("[" &Date()&" - "&Time() &"]  Installed New Hotfixes.")

objFile.Close
LogFile.Close
WScript.Echo "New Ms - Hotfixes are Installed. Please Check the log file for more information on errors and issues"
Wscript.quit(0)

'--------------------------------------------------------------------------------  Functions -----------------------------------------------------------------------------------------
'Install the hotfixes
Function GetAllHotFixes(ByVal hfnames)
	Temp = Split( hfnames,"|",-1,1)

	' if there are no command line arguments mentioned, use "/passive /nobackup /norestart" as default
	If ArrayLength(Temp) = 1  Then 
		CmdArgs = " /passive /nobackup /norestart"
	Else
		CmdArgs = Temp(1)
	End If
	
	hotfixes = Split(Temp(0),"::",-1,1)	
	
	j = ArrayLength(hotfixes)
	
	If j > 0 then
	' if there are no pre-requisites
		If j = 1 then 
			'check if the hotfix is installed or not
			if CheckIfInstalled(hotfixes(j-1)) <> 0 then
				Call InstallHotFix(hotfixes(j-1) ,CmdArgs)
			end if
		else
		' Check if each pre-requisite is installed. If not exit
			for each x in hotfixes
				if x <> hotfixes(j-1) then
					' If not print error msg and exit the loop.
					if CheckIfInstalled(x) <> 0 then 
						WriteLog("[" &Date()&" - "&Time() &"]  Pre - requisites "& x &" for Hotfix "& hotfixes(j -1)&" not installed for Hotfix " & x )
						Exit For
					end if
				else
					Call InstallHotFix(x ,CmdArgs) 
				end if
			next
		end if
	else
		Call WriteLog("[" &Date()&" - "&Time() &"]  HotfixList.dat file is empty. No Hotfixes Installed!")
	end if 
End Function


Function InstallHotFix(ByVal Hotfix, ByVal CmdArgs)
 	
	if objFSO.FileExists(Hotfix) Then
		
		WriteLog("[" &Date()&" - "&Time() &"]  Installing Hotfix " & Hotfix & ".")
		cmd = Hotfix &" " & CmdArgs
		return_code = oShell.run(cmd, 1, True)
	
	Select case return_code
		Case 0
			Call WriteLog ("[" &Date()&" - "&Time() &"]  MS Hotfix  "& Hotfix &"''  Installed successfully.")
		Case 3010
			Call WriteLog ("[" &Date()&" - "&Time() &"]  MS Hotfix '"& Hotfix &"' Installed successfully and reboot requires.")
		Case Else
			Call WriteLog ("[" &Date()&" - "&Time() &"]  ERROR : Installing MS Hotfix '"& Hotfix &"' failed" )
	end select	
	else
		Call WriteLog("[" &Date()&" - "&Time() &"]  ERROR : Hotfix File '" & Hotfix & "' Not Found")
	End if 
	
	
End Function


' Check if the hotfix is installed or not. If installed return 1 and if not return 0
Function CheckIfInstalled(ByVal Hotfix)
	If RegRead("HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion" _
        & "\HotFix\"&GetHotfixName(Hotfix)&"\Installed") <> "1" Then
		CheckIfInstalled = 1
	else
		Call WriteLog ("[" &Date()&" - "&Time() &"]  Hotfix : " & Hotfix & " is already installed")
		CheckIfInstalled = 0
    End if
End Function


'Get Hotfix name from the Given name
Function GetHotFixName(ByVal Hotfix)
	
	' Split the hotfix using delimiter "-"
	hotfixname = Split(Hotfix,"-",-1,1)	
	
	' Create a regular expression
	Set objRegExpr = New regexp
	objRegExpr.Pattern = "[KB|Q][A-Z0-9]*[_|-][A-Z0-9]*"
	objRegExpr.Global = True
	objRegExpr.IgnoreCase = True
	
	'For each of the split keywords. Check if it matches with the regular expression
	
	For each z in objRegExpr.Execute(Hotfix)
	
		' Again split the match to get the actual Hotfix name
		matches = Split(z,"-",-1,1)
		
		if ArrayLength(matches) > 1 then
			
			' if the matches contain x86  then remove that , if not add the rest to the Hotfix name
			
			if matches(1) <> "x86" then
				GetHotFixName = z
			else
				GetHotFixName = matches(0)
			end if 
		Else
				GetHotFixName = matches(0)
		end If
	Next
end Function 

' Write the Log file
Sub WriteLog(LogMsg)
	LogFile.WriteLine(logMsg)
end Sub

'Read the Registry Value
Function RegRead(ByVal sRegValue)      
      On Error Resume Next
      RegRead = oShell.RegRead(sRegValue)
      ' If the value does not exist, error is raised
      If Err Then
        RegRead = ""
        Err.clear
      End If
      ' If a value is present but uninitialized the RegRead method
      ' returns the input value in Win2k.
      If VarType(RegRead) < vbArray Then
        If RegRead = sRegValue Then
          RegRead = ""
        End If
      End If
      On Error Goto 0
End Function

' To find out the length of the array.
Function ArrayLength(ByVal hotfixarray)
	y = 0
	For each names in hotfixarray
		y = y + 1
	next
	ArrayLength = y 'UBound(hotfixarray) + 1
End Function